/* AMBGRO - program for convering AMBER RST coordinates to GROMACS GRO coordinates */
/* Filip Ryjacek 2002 */

#include <stdio.h>
#include <stdlib.h>
#include <errno.h>
#include <stddef.h>
#include <string.h>
#include <math.h>
#include <mcheck.h>

#include "ambconv.h"


/* read AMBER RST coordinates  */
void read_rst(s_atop* atop)
{
    char* rst_title_char;
    FILE* rst;
    unsigned long int natoms;
        
    unsigned int i;

    /* debug memory allocation */
#ifdef DEBUG_MEM
  mtrace();
#endif
#ifdef DEBUG_RST
  printf("\nopening %s ...\n", rst_filename);
#endif

/* open file */
  rst = open_file(rst_filename, "r");

/* read title */
  rst_title_char=fgets(rst_title, MAX_CHAR_LINE, rst);
  if (rst_title_char == NULL) printerror("Cannot read title from RST!");

/* read number of atoms */
  if (fscanf(rst, " %5ld", &natoms) !=1) printerror("Cannot read number of atoms from RST!"); 

  if (natoms != atop->natom) printerror("different number of atoms in RST and TOP!");

/* read coordinates */
ncoords = (atop->natom*3);
coords = (double*) xmalloc(sizeof(double)*ncoords);
  for (i=0; i < ncoords; i++)
    {
      fscanf(rst, " %le", coords+i);
    }
  
/* if IFBOX>0 read box XX,YY,ZZ */
  if (atop->ifbox > 0)
    {
      printf("box not implemented!\n");
    }

// close file
  close_file(rst);

#ifdef DEBUG_RST
  printf("%s closed\n",rst_filename);
#endif
#ifdef DEBUG_MEM
  muntrace();
#endif
}

