/* AMBGRO - program for convering AMBER topology to GROMACS topology */
/* Filip Ryjacek 2001 */

#include <stdio.h>
#include <stdlib.h>
#include <errno.h>
#include <stddef.h>
#include <string.h>
#include <math.h>
#include <mcheck.h>

#include "ambconv.h"


// find duplicate records
unsigned int unique(char** types, char* type, unsigned int i)
  {
    int j;
    unsigned int uni;
    
      uni = 0;
      types[i] = (char*) xmalloc(5);
      strcpy(types[i], type);
      j = i-1;
	while (j >= 0)
	  {
            // if type is already there, return 1
	    if (strcmp(types[j], types[i]) == 0) { uni = 1; }
	    j--;
	  }
  return uni;
  }


void write_ff_itp(void)
  {
    FILE* groitp;
    
    groitp_filename = (char*) xmalloc(strlen(common_filename)+10);
    groitp_filename = strcpy(groitp_filename, "ff");
    groitp_filename = strcat(groitp_filename, common_filename);
    groitp_filename = strcat(groitp_filename, ".itp");
    
    gronb_filename = (char*) xmalloc(strlen(common_filename)+10);
    gronb_filename = strcpy(gronb_filename, "ff");
    gronb_filename = strcat(gronb_filename, common_filename);
    gronb_filename = strcat(gronb_filename, "nb.itp");

#ifdef DEBUG_GTOP
    printf("write_itp: opening %s...\n", groitp_filename); 
#endif
    
    /* open GROMACS main forcefield file */
    groitp = open_file(groitp_filename, "w");

    /* write data to groitp */
    fprintf(groitp, "#define _FF_USER\n\n[ defaults ]\n");
    fprintf(groitp, ";nbfunc     comb-rule      gen-pairs     fudgeLJ      fudgeQQ\n");
    fprintf(groitp, "      1             2            yes         0.5      0.833333\n\n");
    fprintf(groitp, "#include \"%s\"\n", gronb_filename);

    /* close groff */
    close_file(groitp);

#ifdef DEBUG_GTOP
    printf("write_itp: %s closed\n", groitp_filename); 
#endif
  }

/* write atom information to ff???.atp */
void write_ff_atp(s_atop* atop)
  {
    FILE* groatp;
    unsigned int i;
    char* type;
    char* type_pert;
    double mass;
    char** types;
    unsigned int ekviv;
    
    groatp_filename = (char*) xmalloc(strlen(common_filename+10));
    groatp_filename = strcpy(groatp_filename, "ff");
    groatp_filename = strcat(groatp_filename, common_filename);
    groatp_filename = strcat(groatp_filename, ".atp");

#ifdef DEBUG_GTOP
    printf("\nwrite_itp: opening %s...\n", groatp_filename); 
#endif
    
    groatp = open_file(groatp_filename, "w");

    type = (char*) xmalloc(5);
    type_pert = (char*) xmalloc(5);
    types = (char**) xmalloc((atop->natom)*sizeof(char*));
    
    /* read atom types & masses from AMBER top */
    for(i=0; i < atop->natom; i++)
      {
        /* read isymbl & amass from AMBER top */
	type = get_slice(atop->isymbl, type, 4,4,i);
	mass = atop->amass[i];

        /* don't print ekvivalent atoms */
        ekviv = unique(types, type, i);
        if (unique(types, type, i) == 0)
	  {
            /* print to file */
            fprintf(groatp, "  %4s", types[i]);
            fprintf(groatp, "%10.5f\n", mass);
	  }
       }

    /* perurbation   DISABLED */
     if (atop->ifpert == 9999)  
      {
        type_pert = (char*) xmalloc(5);

        /* read atom types & masses from AMBER top */
        for(i=0; i < atop->natom; i++)
          {
            /* read isymbl & amass from AMBER top */
	    type_pert = get_slice(atop->ismper, type_pert, 4,4,i);
	    /* !!! in PARMF only !!! */
	    mass = atop->almper[i];

            /* don't print ekvivalent atoms */
            ekviv = unique(types, type_pert, i);
            if (ekviv == 0)
	      {
                /* print to file */
                fprintf(groatp, "  %4s", types[i]);
                fprintf(groatp, "%10.5f\n", mass);
	      }
           }
         } 
    close_file(groatp);
    
#ifdef DEBUG_GTOP
    printf("write_itp: %s closed\n", groatp_filename); 
#endif

#ifdef USE_FREE
     free(types);
     free(type);
     free(type_pert);
#endif
  }


/* write atom information to ff???nb.itp  ---------------------------------------------- */
void write_ff_nb(s_atop* atop)
  {
    FILE* gronb;
    /* counters in for-cycles */
    unsigned int i,j;
    /* parameters */
    double mass, charge;
    /* LJ params AMBER kcal/mol.A */
    double  c6_a, c12_a, rmin_a, epsilon_a;
    /* LJ params GROMACS kJ/mol.nm */
    double  c6_g, c12_g, rmin_g, epsilon_g, sigma_g;
    /* atom type */
    char* type1;
    char* type2;
    char* type3;
    char* type4;
    /* nonbonded indexes */
    unsigned int iac_i, iac_j, ico_index, ico;
    /* ekvivalent atom types array */
    char** types;
    unsigned int ekviv;

#ifdef DEBUG_GTOP
    printf("\nwrite_itp: opening %s...\n", gronb_filename); 
#endif
    gronb = open_file(gronb_filename, "w");


    /* allocate space for atom types from ISYMBL */    
    type1 = (char*) xmalloc(5);       
    type2 = (char*) xmalloc(5);       
    type3 = (char*) xmalloc(5);       
    type4 = (char*) xmalloc(5);       

    fprintf(gronb, "[ atomtypes ]\n; name        mass      charge    ptype               sigma             epsilon\n");

    types = (char**) xmalloc(2*(atop->natom)*sizeof(char*));

    /* read atom types & masses from AMBER top */
    for(i=0; i < atop->natom; i++)
      {
        /* read isymbl ,amass and charge from AMBER top */
	type1 = get_slice(atop->isymbl, type1, 4,4,i);
	mass = atop->amass[i];
        charge = atop->chrg[i];

        /* don't print ekvivalent types */
        ekviv = unique(types, type1, i);
	if ( ekviv != 0 )
	  {
	    continue;
	  }

        iac_i = atop->iac[i];
        for(j=0; j <= i; j++)
          {
            iac_j = atop->iac[j];
            /* get index to ico array, see AMBER Appendices.doc */
            ico_index = atop->ntypes*(iac_i-1) + iac_j;
            /* array ico has bounds: 0, (ntypes^2)-1 */
            ico_index--;
            ico = atop->ico[ico_index];
            /* ico is index to arrays cn1, cn2 */
            ico--;
            c12_a = atop->cn1[ico]; /* A */
            c6_a  = atop->cn2[ico]; /* C */

            if (i == j)
	      {
	      // get rmin+epsilon in kcal/mol.A
	      rmin_a = pow(2.0 * c12_a/c6_a, 1.0/6.0) * 0.5;
	      epsilon_a = (c6_a*c6_a)/(4.0*c12_a);
	      // convert C12+C6 to kj/mol.nm
	      c12_g = c12_a * conv * 1.000e-12;
	      c6_g  = c6_a  * conv * 1.000e-6;
	      // get rmin+epsilon in kJ/mol.nm
              rmin_g = rmin_a * 0.100000;
              epsilon_g = epsilon_a * conv;

              sigma_g = (rmin_g/1.12246) * 2.00000;
	      

//	      rmin_g = pow(2.0 * c12_g/c6_g, 1.0/6.0) * 0.5;
//	      epsilon_g = (c6_g*c6_g)/(4.0*c12_g);
/*              // convert to kcal/mol.A
              rmin_g = rmin_g*10.0;	      
              epsilon_g = epsilon_g/conv;
*/
	      if ((c12_a < 1e-15) || (c6_a < 1e-15)) { sigma_g = epsilon_g = rmin_a = epsilon_a = 0.0;}

	      fprintf(gronb, "%8s%10.4f%12.5f%9s%20.6e%20.6e", type1, mass, charge, "A", sigma_g, epsilon_g);
	      fprintf(gronb, " ; %8.2f%8.4f\n", rmin_a, epsilon_a);

              if (tinker == 0) {
	      fprintf(tintop, "vdw%10d%16.4f%16.4f   # %2s\n", iac_i ,rmin_a, epsilon_a, type1);
	      }
              }
            }
      }

    if (atop->ifpert > 0)
    {
    // repeat for perturbed atoms
    fprintf(gronb, "; perturbed atoms\n");
    
    for(i=0; i < atop->natom; i++)
      {
        // read perturbed atom type (isymbl) from AMBER top
	type1 = get_slice(atop->ismper, type1, 4,4,i);

        // if type1 from atop->ismper is also in atop->isymbl, skip rest of the cycle
	// because is already there 
	type2 = get_slice(atop->isymbl, type2, 4,4,i);

        if (strcmp(type1, type2) == 0) { continue; }

	mass = atop->almper[i];
        charge = atop->cgper[i];

        /* don't print ekvivalent types */
        ekviv = unique(types, type1, i);
	if ( ekviv != 0 )
	  {
	    continue;
	  }

        iac_i = atop->iacper[i];
        for(j=0; j <= i; j++)
          {
            iac_j = atop->iacper[j];
            /* get index to ico array, see AMBER Appendices.doc */
            ico_index = atop->ntypes*(iac_i-1) + iac_j;
            /* array ico has bounds: 0, (ntypes^2)-1 */
            ico_index--;
            ico = atop->ico[ico_index];
            /* ico is index to arrays cn1, cn2 */
            ico--;
            c12_a = atop->cn1[ico]; /* A */
            c6_a  = atop->cn2[ico]; /* C */

            if (i == j)
	      {
	      // get rmin+epsilon in kcal/mol.A
	      rmin_a = pow(2.0 * c12_a/c6_a, 1.0/6.0) * 0.5;
	      epsilon_a = (c6_a*c6_a)/(4.0*c12_a);
	      // convert C12+C6 to kj/mol.nm
	      c12_g = c12_a * conv * 1.000e-12;
	      c6_g  = c6_a  * conv * 1.000e-6;
	      // get rmin+epsilon in kJ/mol.nm
              rmin_g = rmin_a * 0.100000;
              epsilon_g = epsilon_a * conv;

              // convert rmin to sigma
              sigma_g = (rmin_g/1.12246) * 2.00000;

/*	      rmin_g = pow(2.0 * c12_g/c6_g, 1.0/6.0) * 0.5;
/	      epsilon_g = (c6_g*c6_g)/(4.0*c12_g);
              // convert to kcal/mol.A
              rmin_g = rmin_g*10.0;	      
              epsilon_g = epsilon_g/conv;
*/
	      if ((c12_a < 1e-15) || (c6_a < 1e-15)) { sigma_g = epsilon_g = rmin_a = epsilon_a = 0.0;}

	      fprintf(gronb, "%8s%10.4f%12.5f%9s%20.6e%20.6e", type1, mass, charge, "A", sigma_g, epsilon_g);
	      fprintf(gronb, " ; %8.2f%8.4f\n", rmin_a, epsilon_a);
              }
            }
         }
      }



    // repeat for pair interactions
//    fprintf(gronb, "\n[ nonbond_params ]\n;    i   j func                  c6                 c12\n");
/*
    for(i=0; i < atop->natom; i++)
      {
        // read isymbl & amass from AMBER top
	type1 = get_slice(atop->isymbl, type1, 4,4,i);
        iac_i = atop->iac[i];

        for(j=0; j <= i; j++)
          {
	    type2 = get_slice(atop->isymbl, type2, 4,4,j);
            iac_j = atop->iac[j];
            // get index to ico array, see AMBER Appendices.doc
            ico_index = atop->ntypes*(iac_i-1) + iac_j;
            // array ico has bounds: 0, (ntypes^2)-1
            ico_index--;
            ico = atop->ico[ico_index];
            // ico is index to arrays cn1, cn2
            ico--;
            c12_a = atop->cn1[ico]; // A
            c6_a  = atop->cn2[ico]; // C

            // convert to kj/mol.nm
            c12_g = c12_a * conv * 1.0e-12;
            c6_g  = c6_a  * conv * 1.0e-6;
	    
           if (nbpairs == 0 && ekviv == 0)
	     {
//               fprintf(gronb, "   %4s%4s%4s%20.5e%20.5e\n", type1, type2, "1", c6_g, c12_g);
	     }
          } 
      }
*/
    // pairtypes not used
//    fprintf(gronb, "\n[ pairtypes ]\n; not used\n\n");

    // finalize
//#ifdef USE_FREE
    free(type1);
    free(type2);
    free(type3);
    free(type4);
    free(types);
//#endif

    close_file(gronb);

#ifdef DEBUG_GTOP
    printf("write_itp: %s closed\n", gronb_filename); 
#endif

  }


void write_itp(s_atop* atop) /* --------------------------------------------- */
{

/* debug memory allocation */
#ifdef DEBUG_MEM
  mtrace();
#endif

if (tinker == 0) {
#ifdef DEBUG_GTOP
    printf("\nwrite_itp: opening %s...\n", tintop_filename); 
#endif
    tintop = open_file(tintop_filename, "a");
}


write_ff_itp();
// NOT USED write_ff_atp(atop);
write_ff_nb(atop);

if (tinker ==0) {
  close_file(tintop);

#ifdef DEBUG_GTOP
    printf("write_itp: %s closed.\n", tintop_filename);
#endif
}

#ifdef DEBUG_MEM
  muntrace();
#endif

}

