/* AMBGRO - program for convering AMBER RST coordinates to GROMACS GRO coordinates */
/* Filip Ryjacek 2002 */

#include <stdio.h>
#include <stdlib.h>
#include <errno.h>
#include <stddef.h>
#include <string.h>
#include <mcheck.h>
#include <math.h>

#include "ambconv.h"

void insert_bond(unsigned int a, unsigned int b, unsigned int* connect, FILE* xyz);

// write TINER .xyz file
void write_xyz(s_atop* atop)
  {
    FILE* xyz;
    // atom type
    char* atomname;
    unsigned int i, j;
    unsigned int atom1, atom2;
    unsigned int neighbour;
    // coords
    double x, y, z;
//    unsigned int radek[6];
    unsigned int* connect;

// debug memory allocation
#ifdef DEBUG_MEM
  mtrace();
#endif

#ifdef DEBUG_TTOP
  printf("\nwrite_xyz: opening %s...\n", xyz_filename);
#endif

// open file
xyz = open_file(xyz_filename, "w");

// allocate space    
atomname = (char*) xmalloc(5);
connect = (unsigned int*) xmalloc((atop->natom)*6*sizeof(unsigned int));

// fill array with zeroes
for (i=0; i < (atop->natom)*6; i++) {
  *(connect+i) = (unsigned int) 0;
}

// fill connectivity array: bonds without hydrogen
for (j=0; j < atop->mbona;j++) {
  atom1 = atop->ib[j];
  atom2 = atop->jb[j];

  insert_bond(atom1, atom2, connect, xyz);
  insert_bond(atom2, atom1, connect, xyz);
}

// fill connectivity array: bonds with hydrogen
for (j=0; j < atop->nbonh;j++) {
  atom1 = atop->ibh[j];
  atom2 = atop->jbh[j];

  insert_bond(atom1, atom2, connect, xyz);
  insert_bond(atom2, atom1, connect, xyz);
}

// write number of atoms
fprintf(xyz,"%6i\n", atop->natom);

        for ( i = 0; i < atop->natom; i++)
          {
            // get atom name
            atomname = get_slice(atop->igraph, atomname, 4, 4, i);
	    	    
            x = coords[i*3];
	    y = coords[(i*3)+1];
	    z = coords[(i*3)+2];

            // set the same precision as in .gro or .lacc_rst files
            x = x*100.0;
            y = y*100.0;
            z = z*100.0;

	    x = rint(x)*0.01;
	    y = rint(y)*0.01;
	    z = rint(z)*0.01;

            fprintf(xyz, "%6i%5s%12.6f%12.6f%12.6f%6i", i+1, atomname, x, y, z, i+1);
	    for (j=0; j<6; j++) {
	      neighbour = *(connect+(i*6)+j);
	        if (neighbour !=0) {
	          fprintf(xyz, "%6i", neighbour);
	        }
	    }
	    fprintf(xyz, "\n");
          }



// close file and free memory
close_file(xyz);

#ifdef DEBUG_TTOP
  printf("write_xyz: %s closed\n", xyz_filename);
#endif

//#ifdef USE_FREE
free(atomname);
free(connect);
// from read_rst()
free(coords);
//#endif

#ifdef DEBUG_MEM
  muntrace();
#endif

}

// insert bond for atom a (line a) at firt free position
void insert_bond(unsigned int a, unsigned int b, unsigned int* connect, FILE* xyz) {
  unsigned int i;
  unsigned int position;

//  fprintf(xyz,"inserting connectivity for atom %4d...\n",a);  
  for (i=0; i<6; i++) {
    position = *(connect+((a-1)*6)+i);
    if (position != 0) {
//      fprintf(xyz,"na %4d je obsazeno (%d)\n",i, position);
    }
    else {
//      fprintf(xyz,"na %4d je VOLNO! (%d)\n",i, position);
      *(connect+((a-1)*6)+i) = b;
      break;
    }
  }
}
